@@ -30,21 +30,6 @@ helpers:
     C{docutils.writers.html4css1.Writer}, since those settings will
     be used when we actually write the docstring to html.
 
-Using C{ParsedRstDocstring}s
-============================
-
-C{ParsedRstDocstring}s support all of the methods defined by
-C{ParsedDocstring}; but only the following four methods have
-non-default behavior:
-
-  - L{to_html()<ParsedRstDocstring.to_html>} uses an
-    L{_EpydocHTMLTranslator} to translate the C{ParsedRstDocstring}'s
-    document into an HTML segment.
-  - L{split_fields()<ParsedRstDocstring.split_fields>} uses a
-    L{_SplitFieldsTranslator} to divide the C{ParsedRstDocstring}'s
-    document into its main body and its fields.  Special handling
-    is done to account for consolidated fields.
-
 @var CONSOLIDATED_FIELDS: A dictionary encoding the set of
 'consolidated fields' that can be used.  Each consolidated field is
 marked by a single tag, and contains a single bulleted list, where
@@ -67,16 +52,17 @@ from docutils.readers.standalone import Reader as StandaloneReader
 from docutils.utils import new_document
 from docutils.nodes import NodeVisitor, SkipNode
 from docutils.frontend import OptionParser
-from docutils.parsers.rst import directives, roles
+from docutils.parsers.rst import directives
 import docutils.nodes
 import docutils.transforms.frontmatter
-import docutils.transforms
 import docutils.utils
 
-from pydoctor.epydoc.markup import Field, ParseError, ParsedDocstring
+from twisted.web.template import tags
+from pydoctor.epydoc.doctest import colorize_codeblock, colorize_doctest
+from pydoctor.epydoc.markup import (
+    Field, ParseError, ParsedDocstring, flatten, html2stan
+)
 from pydoctor.epydoc.markup.plaintext import ParsedPlaintextDocstring
-from pydoctor.epydoc.util import plaintext_to_html
-from pydoctor.epydoc.markup.doctest import doctest_to_html, HTMLDoctestColorizer
 
 #: A dictionary whose keys are the "consolidated fields" that are
 #: recognized by epydoc; and whose values are the corresponding epydoc
@@ -117,7 +103,12 @@ def parse_docstring(docstring, errors):
                    settings_overrides={'report_level':10000,
                                        'halt_level':10000,
                                        'warning_stream':None})
-    return ParsedRstDocstring(writer.document)
+
+    document = writer.document
+    visitor = _SplitFieldsTranslator(document, errors)
+    document.walk(visitor)
+
+    return ParsedRstDocstring(document, visitor.fields)
 
 class OptimizedReporter(docutils.utils.Reporter):
     """A reporter that ignores all debug messages.  This is used to
@@ -135,35 +126,22 @@ class ParsedRstDocstring(ParsedDocstring):
         docstring.
     @type _document: C{docutils.nodes.document}
     """
-    def __init__(self, document):
+    def __init__(self, document, fields):
         """
         @type document: C{docutils.nodes.document}
         """
         self._document = document
 
-        # The default document reporter and transformer are not
-        # pickle-able; so replace them with stubs that are.
         document.reporter = OptimizedReporter(
             document.reporter.source, 'SEVERE', 'SEVERE', '')
-        document.transformer = docutils.transforms.Transformer(document)
 
-    def split_fields(self, errors=None):
-        # Inherit docs
-        if errors is None: errors = []
-        visitor = _SplitFieldsTranslator(self._document, errors)
-        self._document.walk(visitor)
-        if len(self._document.children) > 0:
-            return self, visitor.fields
-        else:
-            return None, visitor.fields
+        ParsedDocstring.__init__(self, fields)
 
-    def to_html(self, docstring_linker, directory=None,
-                docindex=None, context=None):
+    def to_stan(self, docstring_linker):
         # Inherit docs
-        visitor = _EpydocHTMLTranslator(self._document, docstring_linker,
-                                        directory, docindex, context)
+        visitor = _EpydocHTMLTranslator(self._document, docstring_linker)
         self._document.walkabout(visitor)
-        return ''.join(visitor.body)
+        return html2stan(''.join(visitor.body))
 
     def __repr__(self): return '<ParsedRstDocstring: ...>'
 
@@ -274,17 +252,18 @@ class _SplitFieldsTranslator(NodeVisitor):
                         # Use a @newfield to let it be displayed as-is.
                         if tagname.lower() not in self._newfields:
                             newfield = Field('newfield', tagname.lower(),
-                                             ParsedPlaintextDocstring(tagname))
+                                             ParsedPlaintextDocstring(tagname),
+                                             node.line - 1)
                             self.fields.append(newfield)
                             self._newfields.add(tagname.lower())
 
-        self._add_field(tagname, arg, fbody)
+        self._add_field(tagname, arg, fbody, node.line)
 
-    def _add_field(self, tagname, arg, fbody):
+    def _add_field(self, tagname, arg, fbody, lineno):
         field_doc = self.document.copy()
         for child in fbody: field_doc.append(child)
-        field_pdoc = ParsedRstDocstring(field_doc)
-        self.fields.append(Field(tagname, arg, field_pdoc))
+        field_pdoc = ParsedRstDocstring(field_doc, ())
+        self.fields.append(Field(tagname, arg, field_pdoc, lineno - 1))
 
     def visit_field_list(self, node):
         # Remove the field list from the tree.  The visitor will still walk
@@ -357,7 +336,7 @@ class _SplitFieldsTranslator(NodeVisitor):
                         )
 
             # Wrap the field body, and add a new field
-            self._add_field(tagname, arg, fbody)
+            self._add_field(tagname, arg, fbody, fbody[0].line)
 
     def handle_consolidated_definition_list(self, items, tagname):
         # Check the list contents.
@@ -369,7 +348,6 @@ class _SplitFieldsTranslator(NodeVisitor):
         for item in items:
             n += 1
             if (item.tagname != 'definition_list_item' or len(item) < 2 or
-                item[0].tagname != 'term' or
                 item[-1].tagname != 'definition'):
                 raise ValueError('bad definition list (bad child %d).' % n)
             if len(item) > 3:
@@ -386,12 +364,13 @@ class _SplitFieldsTranslator(NodeVisitor):
         for item in items:
             # The basic field.
             arg = item[0][0].astext()
+            lineno = item[0].line
             fbody = item[-1]
-            self._add_field(tagname, arg, fbody)
+            self._add_field(tagname, arg, fbody, lineno)
             # If there's a classifier, treat it as a type.
             if len(item) == 3:
                 type_descr = item[1]
-                self._add_field('type', arg, type_descr)
+                self._add_field('type', arg, type_descr, lineno)
 
     def unknown_visit(self, node):
         'Ignore all unknown nodes'
@@ -400,12 +379,8 @@ _TARGET_RE = re.compile(r'^(.*?)\s*<(?:URI:|URL:)?([^<>]+)>$')
 
 class _EpydocHTMLTranslator(HTMLTranslator):
     settings = None
-    def __init__(self, document, docstring_linker, directory,
-                 docindex, context):
+    def __init__(self, document, docstring_linker):
         self._linker = docstring_linker
-        self._directory = directory
-        self._docindex = docindex
-        self._context = context
 
         # Set the document's settings.
         if self.settings is None:
@@ -421,9 +396,14 @@ class _EpydocHTMLTranslator(HTMLTranslator):
         m = _TARGET_RE.match(node.astext())
         if m: text, target = m.groups()
         else: target = text = node.astext()
-        text = plaintext_to_html(text)
-        xref = self._linker.translate_identifier_xref(target, text)
-        self.body.append(xref)
+        label = tags.code(text)
+        try:
+            url = self._linker.resolve_identifier_xref(target)
+        except LookupError:
+            xref = label
+        else:
+            xref = tags.a(label, href=url)
+        self.body.append(flatten(xref))
         raise SkipNode()
 
     def should_be_compact_paragraph(self, node):
@@ -481,22 +461,11 @@ class _EpydocHTMLTranslator(HTMLTranslator):
     def visit_doctest_block(self, node):
         pysrc = node[0].astext()
         if node.get('codeblock'):
-            self.body.append(HTMLDoctestColorizer().colorize_codeblock(pysrc))
+            self.body.append(flatten(colorize_codeblock(pysrc)))
         else:
-            self.body.append(doctest_to_html(pysrc))
+            self.body.append(flatten(colorize_doctest(pysrc)))
         raise SkipNode()
 
-    def visit_emphasis(self, node):
-        # Generate a corrent index term anchor
-        if 'term' in node.get('classes') and node.children:
-            doc = self.document.copy()
-            doc[:] = [node.children[0].copy()]
-            self.body.append(
-                self._linker.translate_indexterm(ParsedRstDocstring(doc)))
-            raise SkipNode()
-
-        HTMLTranslator.visit_emphasis(self, node)
-
 def python_code_directive(name, arguments, options, content, lineno,
                           content_offset, block_text, state, state_machine):
     """
@@ -516,14 +485,3 @@ python_code_directive.arguments = (0, 0, 0)
 python_code_directive.content = True
 
 directives.register_directive('python', python_code_directive)
-
-def term_role(name, rawtext, text, lineno, inliner,
-            options={}, content=[]):
-
-    text = docutils.utils.unescape(text)
-    node = docutils.nodes.emphasis(rawtext, text, **options)
-    node.attributes['classes'].append('term')
-
-    return [node], []
-
-roles.register_local_role('term', term_role)
