@@ -4,49 +4,65 @@ Convert epydoc markup into renderable content.
 
 from __future__ import print_function
 
-from six.moves import builtins
-from six.moves.urllib.parse import quote
-import exceptions
-import inspect
+import astor
+
+from importlib import import_module
 import itertools
 import os
-import re
 import sys
 
-from twisted.web.template import Tag, tags, XMLString
-
 from pydoctor import model
-from pydoctor.epydoc.markup import DocstringLinker
+from pydoctor.epydoc.markup import ParseError
+from six import string_types
+from six.moves import builtins
+from six.moves.urllib.parse import quote
+from twisted.web.template import Tag, tags
+from pydoctor.epydoc.markup import DocstringLinker, ParsedDocstring
+import pydoctor.epydoc.markup.plaintext
 
 
-STDLIB_DIR = os.path.dirname(os.__file__)
-STDLIB_URL = 'http://docs.python.org/library/'
+def _find_stdlib_dir():
+    """Find the standard library location for the currently running
+    Python interpreter.
+    """
+
+    # When running in a virtualenv, when some (but not all) modules
+    # may be symlinked. We want the actual installation location of
+    # the standard library, not the location of the virtualenv.
+    os_mod_path = os.__file__
+    if os_mod_path.endswith('.pyc') or os_mod_path.endswith('.pyo'):
+        os_mod_path = os_mod_path[:-1]
+    return os.path.dirname(os.path.realpath(os_mod_path))
+
+STDLIB_DIR = _find_stdlib_dir()
+STDLIB_URL = 'https://docs.python.org/3/library/'
 
 
 def link(o):
-    return o.system.urlprefix + quote(o.fullName()+'.html')
+    return quote(o.fullName()+'.html')
 
 
-def get_parser(formatname):
+def get_parser(obj):
+    formatname = obj.system.options.docformat
     try:
-        mod = __import__('pydoctor.epydoc.markup.' + formatname,
-                         globals(), locals(), ['parse_docstring'])
+        mod = import_module('pydoctor.epydoc.markup.' + formatname)
     except ImportError as e:
-        return None, e
-    else:
-        return mod.parse_docstring, None
+        msg = 'Error trying to import %r parser:\n\n    %s: %s\n\nUsing plain text formatting only.'%(
+            formatname, e.__class__.__name__, e)
+        obj.system.msg('epydoc2stan', msg, thresh=-1, once=True)
+        mod = pydoctor.epydoc.markup.plaintext
+    return mod.parse_docstring
 
 
-def boringDocstring(doc, summary=False):
-    """Generate an HTML representation of a docstring in a really boring way.
-    """
-    # inspect.getdoc requires an object with a __doc__ attribute, not
-    # just a string :-(
-    if doc is None or not doc.strip():
-        return '<pre class="undocumented">Undocumented</pre>'
-    def crappit(): pass
-    crappit.__doc__ = doc
-    return (tags.pre, tags.tt)[bool(summary)](inspect.getdoc(crappit))
+def get_docstring(obj):
+    for source in obj.docsources():
+        doc = source.docstring
+        if doc:
+            return doc, source
+        if doc is not None:
+            # Treat empty docstring as undocumented.
+            return None, source
+    return None, None
 
 
 def stdlib_doc_link_for_name(name):
@@ -64,10 +80,11 @@ def stdlib_doc_link_for_name(name):
     part0 = parts[0]
     if part0 in builtins.__dict__ and not part0.startswith('__'):
         bltin = builtins.__dict__[part0]
-        if part0 in exceptions.__dict__:
-            return STDLIB_URL + 'exceptions.html#exceptions.' + name
-        elif isinstance(bltin, type):
-            return STDLIB_URL + 'stdtypes.html#' + name
+        if isinstance(bltin, type):
+            if issubclass(bltin, BaseException):
+                return STDLIB_URL + 'exceptions.html#' + name
+            else:
+                return STDLIB_URL + 'stdtypes.html#' + name
         elif callable(bltin):
             return STDLIB_URL + 'functions.html#' + name
         else:
@@ -80,24 +97,17 @@ class _EpydocLinker(DocstringLinker):
     def __init__(self, obj):
         self.obj = obj
 
-    def translate_indexterm(self, something):
-        # X{foobar} is meant to put foobar in an index page (like, a
-        # proper end-of-the-book index). Should we support that? There
-        # are like 2 uses in Twisted.
-        return de_p(something.to_html(self))
-
-    def _objLink(self, obj, prettyID):
-        if obj.documentation_location == model.DocLocation.PARENT_PAGE:
+    def _objLink(self, obj):
+        if obj.documentation_location is model.DocLocation.PARENT_PAGE:
             p = obj.parent
             if isinstance(p, model.Module) and p.name == '__init__':
                 p = p.parent
-            linktext = link(p) + '#' + quote(obj.name)
-        elif obj.documentation_location == model.DocLocation.OWN_PAGE:
-            linktext = link(obj)
+            return link(p) + '#' + quote(obj.name)
+        elif obj.documentation_location is model.DocLocation.OWN_PAGE:
+            return link(obj)
         else:
             raise AssertionError(
                 "Unknown documentation_location: %s" % obj.documentation_location)
-        return '<a href="%s"><code>%s</code></a>'%(linktext, prettyID)
 
     def look_for_name(self, name, candidates):
         part0 = name.split('.')[0]
@@ -111,11 +121,11 @@ class _EpydocLinker(DocstringLinker):
         if len(potential_targets) == 1:
             return potential_targets[0]
         elif len(potential_targets) > 1:
-            self.obj.system.msg(
-                "translate_identifier_xref", "%s:%s ambiguous ref to %s, could be %s" % (
-                    self.obj.fullName(), self.obj.linenumber, name,
-            ', '.join([ob.fullName() for ob in potential_targets])),
-                thresh=-1)
+            self.obj.report(
+                "ambiguous ref to %s, could be %s" % (
+                    name,
+                    ', '.join(ob.fullName() for ob in potential_targets)),
+                section='resolve_identifier_xref')
         return None
 
     def look_for_intersphinx(self, name):
@@ -126,57 +136,23 @@ class _EpydocLinker(DocstringLinker):
         """
         return self.obj.system.intersphinx.getLink(name)
 
-    def translate_identifier_xref(self, fullID, prettyID):
-        """Figure out what ``L{fullID}`` should link to.
+    def resolve_identifier_xref(self, fullID):
 
-        There is a lot of DWIM here.  The order goes:
+        # There is a lot of DWIM here. Look for a global match first,
+        # to reduce the chance of a false positive.
 
-          1. Check if fullID refers to an object by Python name resolution in
-             our context.
-
-          2. Walk up the object tree and see if fullID refers to an object by
-             Python name resolution in each context.
-
-          3. Check if fullID is the fullName of an object.
-
-          4. Check to see if fullID names a builtin or standard library
-             module.
-
-          4. Walk up the object tree again and see if fullID refers to an
-             object in an "uncle" object.  (So if p.m1 has a class C, the
-             docstring for p.m2 can say L{C} to refer to the class in m1).  If
-             at any level fullID refers to more than one object, complain.
-
-          5. Examine every module and package in the system and see if fullID
-             names an object in each one.  Again, if more than one object is
-             found, complain.
-
-        """
-        src = self.obj
-        while src is not None:
-            target = src.resolveName(fullID)
-            if target is not None:
-                return self._objLink(target, prettyID)
-            src = src.parent
+        # Check if fullID is the fullName of an object.
         target = self.obj.system.objForFullName(fullID)
         if target is not None:
-            return self._objLink(target, prettyID)
+            return self._objLink(target)
+
+        # Check to see if fullID names a builtin or standard library module.
         fullerID = self.obj.expandName(fullID)
         linktext = stdlib_doc_link_for_name(fullerID)
         if linktext is not None:
-            return '<a href="%s"><code>%s</code></a>'%(linktext, prettyID)
-        src = self.obj
-        while src is not None:
-            target = self.look_for_name(fullID, src.contents.values())
-            if target is not None:
-                return self._objLink(target, prettyID)
-            src = src.parent
-        target = self.look_for_name(fullID, itertools.chain(
-            self.obj.system.objectsOfType(model.Module),
-            self.obj.system.objectsOfType(model.Package)))
-        if target is not None:
-            return self._objLink(target, prettyID)
+            return linktext
 
+        # Check if the fullID exists in an intersphinx inventory.
         target = self.look_for_intersphinx(fullerID)
         if not target:
             # FIXME: https://github.com/twisted/pydoctor/issues/125
@@ -184,14 +160,50 @@ class _EpydocLinker(DocstringLinker):
             # try our luck with fullID.
             target = self.look_for_intersphinx(fullID)
         if target:
-            return '<a href="%s"><code>%s</code></a>'%(target, prettyID)
+            return target
 
-        self.obj.system.msg(
-            "translate_identifier_xref", "%s:%s invalid ref to '%s' "
-            "resolved as '%s'" % (
-                self.obj.fullName(), self.obj.linenumber, fullID, fullerID),
-            thresh=-1)
-        return '<code>%s</code>'%(prettyID,)
+        # Since there was no global match, go look for the name in the
+        # context where it was used.
+
+        # Check if fullID refers to an object by Python name resolution
+        # in our context. Walk up the object tree and see if fullID refers
+        # to an object by Python name resolution in each context.
+        src = self.obj
+        while src is not None:
+            target = src.resolveName(fullID)
+            if target is not None:
+                return self._objLink(target)
+            src = src.parent
+
+        # Walk up the object tree again and see if fullID refers to an
+        # object in an "uncle" object.  (So if p.m1 has a class C, the
+        # docstring for p.m2 can say L{C} to refer to the class in m1).
+        # If at any level fullID refers to more than one object, complain.
+        src = self.obj
+        while src is not None:
+            target = self.look_for_name(fullID, src.contents.values())
+            if target is not None:
+                return self._objLink(target)
+            src = src.parent
+
+        # Examine every module and package in the system and see if fullID
+        # names an object in each one.  Again, if more than one object is
+        # found, complain.
+        target = self.look_for_name(fullID, itertools.chain(
+            self.obj.system.objectsOfType(model.Module),
+            self.obj.system.objectsOfType(model.Package)))
+        if target is not None:
+            return self._objLink(target)
+
+        if fullID == fullerID:
+            self.obj.report(
+                "invalid ref to '%s' not resolved" % (fullID,),
+                section='resolve_identifier_xref')
+        else:
+            self.obj.report(
+                "invalid ref to '%s' resolved as '%s'" % (fullID, fullerID),
+                section='resolve_identifier_xref')
+        raise LookupError(fullID)
 
 
 class FieldDesc(object):
@@ -210,7 +222,7 @@ class FieldDesc(object):
         return body
     def __repr__(self):
         contents = []
-        for k, v in self.__dict__.iteritems():
+        for k, v in self.__dict__.items():
             contents.append("%s=%r"%(k, v))
         return "<%s(%s)>"%(self.__class__.__name__, ', '.join(contents))
 
@@ -265,46 +277,21 @@ def format_field_list(obj, singular, fields, plural=None):
     return rows
 
 
-_ok_chars = '\r\n\t\f'
-_class = ''
-for _c in map(chr, range(0, 32)):
-    if _c not in _ok_chars:
-        _class += _c
-_control_pat = re.compile('[' + _class + ']')
-
-
-def html2stan(html):
-    """
-    Convert HTML to a Stan structure.
-
-    @param html: A HTML string.
-    @type html: L{unicode} or L{bytes}
-    """
-    if isinstance(html, unicode):
-        html = html.encode('utf8')
-
-    html = _control_pat.sub(lambda m:'\\x%02x' % ord(m.group()), html)
-    html = "<div>" + html + "</div>"
-    html = XMLString(html).load()[0].children
-    if html and html[-1] == u'\n':
-        del html[-1]
-    return html
-
-
 class Field(object):
     """Like pydoctor.epydoc.markup.Field, but without the gross accessor
     methods and with a formatted body."""
     def __init__(self, field, obj):
         self.tag = field.tag()
         self.arg = field.arg()
-        self.body = html2stan(field.body().to_html(_EpydocLinker(obj)))
+        self.lineno = field.lineno
+        self.body = field.body().to_stan(_EpydocLinker(obj))
 
     def __repr__(self):
         r = repr(self.body)
         if len(r) > 25:
             r = r[:20] + '...' + r[-2:]
-        return "<%s %r %r %s>"%(self.__class__.__name__,
-                             self.tag, self.arg, r)
+        return "<%s %r %r %d %s>"%(self.__class__.__name__,
+                             self.tag, self.arg, self.lineno, r)
 
 class FieldHandler(object):
     def __init__(self, obj):
@@ -398,10 +385,11 @@ class FieldHandler(object):
         self.sinces.append(field)
 
     def handleUnknownField(self, field):
-        self.obj.system.msg(
-            'epydoc2stan',
-            'found unknown field on %r: %r'%(self.obj.fullName(), field),
-            thresh=-1)
+        self.obj.report(
+            'unknown field "%s"' % field.tag,
+            lineno_offset=field.lineno,
+            section='docstring'
+            )
         self.add_info(self.unknowns, field)
 
     def handle(self, field):
@@ -442,136 +430,176 @@ class FieldHandler(object):
         return tags.table(class_='fieldTable')(r)
 
 
-def de_p(s):
-    if s.startswith('<p>') and s.endswith('</p>\n'):
-        s = s[3:-5] # argh reST
-    if s.endswith('\n'):
-        s = s[:-1]
+def reportErrors(obj, errs):
+    if errs and obj.fullName() not in obj.system.docstring_syntax_errors:
+        obj.system.docstring_syntax_errors.add(obj.fullName())
+
+        for err in errs:
+            lineno_offset = 0
+            if isinstance(err, string_types):
+                descr = err
+            elif isinstance(err, ParseError):
+                descr = err.descr()
+                lineno_offset = err.linenum() - 1
+            else:
+                raise TypeError(type(err).__name__)
+
+            obj.report(
+                'bad docstring: ' + descr,
+                lineno_offset=lineno_offset,
+                section='docstring'
+                )
+
+
+def parse_docstring(obj, doc, source):
+    """Parse a docstring.
+    @rtype: L{ParsedDocstring}
+    """
+
+    parser = get_parser(obj)
+    errs = []
+    try:
+        pdoc = parser(doc, errs)
+    except Exception as e:
+        errs.append('%s: %s' % (e.__class__.__name__, e))
+        pdoc = None
+    if pdoc is None:
+        pdoc = pydoctor.epydoc.markup.plaintext.parse_docstring(doc, errs)
+    if errs:
+        reportErrors(source, errs)
+    return pdoc
+
+
+def format_docstring(obj):
+    """Generate an HTML representation of a docstring"""
+
+    doc, source = get_docstring(obj)
+
+    # Use cached or split version if possible.
+    pdoc = getattr(obj, 'parsed_docstring', None)
+
+    if pdoc is None:
+        if doc is None:
+            return tags.div(class_='undocumented')("Undocumented")
+        pdoc = parse_docstring(obj, doc, source)
+        obj.parsed_docstring = pdoc
+    elif source is None:
+        # A split field is documented by its parent.
+        source = obj.parent
+
+    try:
+        stan = pdoc.to_stan(_EpydocLinker(source))
+    except Exception as e:
+        errs = ['%s: %s' % (e.__class__.__name__, e)]
+        if doc is None:
+            stan = tags.p(class_="undocumented")('Broken description')
+        else:
+            pdoc_plain = pydoctor.epydoc.markup.plaintext.parse_docstring(doc, errs)
+            stan = pdoc_plain.to_stan(_EpydocLinker(source))
+        reportErrors(source, errs)
+
+    content = [stan] if stan.tagName else stan.children
+    fields = pdoc.fields
+    s = tags.div(*content)
+    if fields:
+        fh = FieldHandler(obj)
+        for field in fields:
+            fh.handle(Field(field, obj))
+        fh.resolve_types()
+        s(fh.format())
     return s
 
 
-def reportErrors(obj, errs):
-    for err in errs:
-        if isinstance(err, str):
-            linenumber = '??'
-            descr = err
+def format_summary(obj):
+    """Generate an shortened HTML representation of a docstring."""
+
+    doc, source = get_docstring(obj)
+    if doc is None:
+        # Attributes can be documented as fields in their parent's docstring.
+        if isinstance(obj, model.Attribute):
+            pdoc = getattr(obj, 'parsed_docstring', None)
         else:
-            linenumber = obj.linenumber + err.linenum()
-            descr = err._descr
-        obj.system.msg(
-            'epydoc2stan2',
-            '%s:%s epytext error %r' % (obj.fullName(), linenumber, descr))
-    if errs and obj.fullName() not in obj.system.epytextproblems:
-        obj.system.epytextproblems.append(obj.fullName())
-        obj.system.msg('epydoc2stan',
-                       'epytext error in %s'%(obj,), thresh=1)
-        p = lambda m:obj.system.msg('epydoc2stan', m, thresh=2)
-        for i, l in enumerate(obj.docstring.splitlines()):
-            p("%4s"%(i+1)+' '+l)
-        for err in errs:
-            p(err)
-
-
-def doc2stan(obj, summary=False, docstring=None):
-    """Generate an HTML representation of a docstring"""
-    if getattr(obj, 'parsed_docstring', None) is not None:
-        r = html2stan(obj.parsed_docstring.to_html(_EpydocLinker(obj)))
-        if getattr(obj, 'parsed_type', None) is not None:
-            r = [r, ' (type: ', html2stan(obj.parsed_type.to_html(_EpydocLinker(obj))), ')']
-        return r, []
-    origobj = obj
-    if isinstance(obj, model.Package):
-        obj = obj.contents['__init__']
-    if docstring is None:
-        doc = None
-        for source in obj.docsources():
-            if source.docstring is not None:
-                doc = source.docstring
-                break
+            pdoc = None
+        if pdoc is None:
+            return format_undocumented(obj)
+        source = obj.parent
     else:
-        source = obj
-        doc = docstring
-    if doc is None or not doc.strip():
-        text = "Undocumented"
-        subdocstrings = {}
-        subcounts = {}
-        for subob in origobj.contents.itervalues():
-            k = subob.kind.lower()
-            subcounts[k] = subcounts.get(k, 0) + 1
-            if subob.docstring is not None:
-                subdocstrings[k] = subdocstrings.get(k, 0) + 1
-        if isinstance(origobj, model.Package):
-            subcounts["module"] -= 1
-        if subdocstrings:
-            plurals = {'class':'classes'}
-            text = "No %s docstring"%origobj.kind.lower()
-            if summary:
-                u = []
-                for k in sorted(subcounts):
-                    u.append("%s/%s %s"%(subdocstrings.get(k, 0), subcounts[k],
-                                         plurals.get(k, k+'s')))
-                text += '; ' + ', '.join(u) + " documented"
-        if summary:
-            return tags.span(class_="undocumented")(text), []
-        else:
-            return tags.div(class_="undocumented")(text), []
-    if summary:
         # Use up to three first non-empty lines of doc string as summary.
         lines = itertools.dropwhile(lambda line: not line.strip(),
                                     doc.split('\n'))
         lines = itertools.takewhile(lambda line: line.strip(), lines)
         lines = [ line.strip() for line in lines ]
         if len(lines) > 3:
-            return tags.span(class_="undocumented")('No summary'), []
-        else:
-            doc = ' '.join(lines)
-    parse_docstring, e = get_parser(obj.system.options.docformat)
-    if not parse_docstring:
-        msg = 'Error trying to import %r parser:\n\n    %s: %s\n\nUsing plain text formatting only.'%(
-            obj.system.options.docformat, e.__class__.__name__, e)
-        obj.system.msg('epydoc2stan', msg, thresh=-1, once=True)
-        return boringDocstring(doc, summary), []
-    errs = []
-    def crappit(): pass
-    crappit.__doc__ = doc
-    doc = inspect.getdoc(crappit)
+            return tags.span(class_='undocumented')("No summary")
+        pdoc = parse_docstring(obj, ' '.join(lines), source)
+
     try:
-        pdoc = parse_docstring(doc, errs)
-    except Exception as e:
-        errs = [e.__class__.__name__ +': ' + str(e)]
-    if errs:
-        reportErrors(source, errs)
-        return boringDocstring(doc, summary), errs
-    pdoc, fields = pdoc.split_fields()
-    if pdoc is not None:
-        try:
-            crap = pdoc.to_html(_EpydocLinker(source))
-        except Exception as e:
-            reportErrors(source, [e.__class__.__name__ +': ' + str(e)])
-            return (boringDocstring(doc, summary),
-                    [e.__class__.__name__ +': ' + str(e)])
+        stan = pdoc.to_stan(_EpydocLinker(source))
+    except Exception:
+        # This problem will likely be reported by the full docstring as well,
+        # so don't spam the log.
+        return tags.span(class_='undocumented')("Broken description")
+
+    content = [stan] if stan.tagName else stan.children
+    if content and isinstance(content[0], Tag) and content[0].tagName == 'p':
+        content = content[0].children
+    return tags.span(*content)
+
+
+def format_undocumented(obj):
+    """Generate an HTML representation for an object lacking a docstring."""
+    subdocstrings = {}
+    subcounts = {}
+    for subob in obj.contents.values():
+        k = subob.kind.lower()
+        subcounts[k] = subcounts.get(k, 0) + 1
+        if subob.docstring is not None:
+            subdocstrings[k] = subdocstrings.get(k, 0) + 1
+    if isinstance(obj, model.Package):
+        subcounts['module'] -= 1
+    if subdocstrings:
+        plurals = {'class': 'classes'}
+        text = "No %s docstring; %s documented" % (
+            obj.kind.lower(),
+            ', '.join(
+                "%s/%s %s" % (subdocstrings.get(k, 0), subcounts[k],
+                              plurals.get(k, k + 's'))
+                for k in sorted(subcounts)
+                )
+            )
     else:
-        crap = ''
-    if isinstance(crap, unicode):
-        crap = crap.encode('utf-8')
-    if summary:
-        if not crap:
-            return (), []
-        stan = html2stan(crap)
-        if len(stan) == 1 and isinstance(stan[0], Tag) and stan[0].tagName == 'p':
-            stan = stan[0].children
-        s = tags.span(stan)
+        text = "Undocumented"
+    return tags.span(class_='undocumented')(text)
+
+
+def type2stan(obj):
+    parsed_type = get_parsed_type(obj)
+    if parsed_type is None:
+        return None
     else:
-        if not crap and not fields:
-            return (), []
-        stan = html2stan(crap)
-        s = tags.div(stan)
-        fh = FieldHandler(obj)
-        for field in fields:
-            fh.handle(Field(field, obj))
-        fh.resolve_types()
-        s(fh.format())
-    return s, []
+        return parsed_type.to_stan(_EpydocLinker(obj))
+
+def get_parsed_type(obj):
+    parsed_type = getattr(obj, 'parsed_type', None)
+    if parsed_type is not None:
+        return parsed_type
+
+    annotation = getattr(obj, 'annotation', None)
+    if annotation is not None:
+        return AnnotationDocstring(annotation)
+
+    return None
+
+
+class AnnotationDocstring(ParsedDocstring):
+
+    def __init__(self, annotation):
+        ParsedDocstring.__init__(self, ())
+        self.annotation = annotation
+
+    def to_stan(self, docstring_linker):
+        src = astor.to_source(self.annotation).strip()
+        return tags.code(src)
 
 
 field_name_to_human_name = {
@@ -582,36 +610,30 @@ field_name_to_human_name = {
 
 
 def extract_fields(obj):
-    if isinstance(obj, model.Package):
-        obj = obj.contents['__init__']
-    if isinstance(obj, model.Function):
-        return []
-    doc = obj.docstring
-    if doc is None or not doc.strip():
-        return []
-    parse_docstring, e = get_parser(obj.system.options.docformat)
-    if not parse_docstring:
-        return []
-    def crappit(): pass
-    crappit.__doc__ = doc
-    doc = inspect.getdoc(crappit)
-    try:
-        pdoc = parse_docstring(doc, [])
-    except Exception:
-        return []
-    pdoc, fields = pdoc.split_fields()
-    if not fields:
-        return []
-    r = []
-    types = {}
-    for field in fields:
-        if field.tag() == 'type':
-            types[field.arg()] = field.body()
-    for field in fields:
-        if field.tag() in ['ivar', 'cvar', 'var']:
-            attrobj = obj.system.Attribute(obj.system, field.arg(), None, obj)
-            attrobj.parsed_docstring = field.body()
-            attrobj.parsed_type = types.get(field.arg())
-            attrobj.kind = field_name_to_human_name[field.tag()]
-            r.append(attrobj)
-    return r
+    doc, source = get_docstring(obj)
+    if doc is None:
+        return
+
+    pdoc = parse_docstring(obj, doc, source)
+    obj.parsed_docstring = pdoc
+
+    for field in pdoc.fields:
+        tag = field.tag()
+        if tag in ['ivar', 'cvar', 'var', 'type']:
+            arg = field.arg()
+            if arg is None:
+                obj.report("Missing field name in @%s" % (tag,),
+                           'docstring', field.lineno)
+                continue
+            attrobj = obj.contents.get(arg)
+            if attrobj is None:
+                attrobj = obj.system.Attribute(obj.system, arg, obj)
+                attrobj.kind = None
+                attrobj.parentMod = obj.parentMod
+                obj.system.addObject(attrobj)
+            attrobj.setLineNumber(source.docstring_lineno + field.lineno)
+            if tag == 'type':
+                attrobj.parsed_type = field.body()
+            else:
+                attrobj.parsed_docstring = field.body()
+                attrobj.kind = field_name_to_human_name[tag]
