@@ -43,14 +43,23 @@ def signature(argspec):
             things.append(regarg)
     if varargname:
         things.append('*%s' % varargname)
-    things += ['%s=%s' % (t[0], t[1]) for t in kwargs]
+
+    things += ['%s=%s' % kwarg for kwarg in kwargs]
     if varkwname:
         things.append('**%s' % varkwname)
     return ', '.join(things)
 
 class DocGetter(object):
     def get(self, ob, summary=False):
-        return epydoc2stan.doc2stan(ob, summary=summary)[0]
+        if summary:
+            return epydoc2stan.format_summary(ob)
+        else:
+            doc = epydoc2stan.format_docstring(ob)
+            typ = epydoc2stan.type2stan(ob)
+            if typ is None:
+                return doc
+            else:
+                return [doc, ' (type: ', typ, ')']
 
 class CommonPage(Element):
 
@@ -124,8 +133,8 @@ class CommonPage(Element):
 
     def children(self):
         return sorted(
-            [o for o in self.ob.orderedcontents if o.isVisible],
-            key=lambda o:-o.privacyClass)
+            [o for o in self.ob.contents.values() if o.isVisible],
+            key=lambda o:-o.privacyClass.value)
 
     def packageInitTable(self):
         return ()
@@ -142,8 +151,8 @@ class CommonPage(Element):
             return ()
 
     def methods(self):
-        return [o for o in self.ob.orderedcontents
-                if o.documentation_location == model.DocLocation.PARENT_PAGE
+        return [o for o in self.ob.contents.values()
+                if o.documentation_location is model.DocLocation.PARENT_PAGE
                 and o.isVisible]
 
     def childlist(self):
@@ -154,7 +163,7 @@ class CommonPage(Element):
             if isinstance(c, model.Function):
                 r.append(FunctionChild(self.docgetter, c, self.functionExtras(c)))
             else:
-                r.append(AttributeChild(self.docgetter, c))
+                r.append(AttributeChild(self.docgetter, c, self.functionExtras(c)))
         return r
 
     def functionExtras(self, data):
@@ -180,15 +189,15 @@ class CommonPage(Element):
 
 class PackagePage(CommonPage):
     def children(self):
-        return sorted([o for o in self.ob.orderedcontents
+        return sorted([o for o in self.ob.contents.values()
                        if o.name != '__init__' and o.isVisible],
-                      key=lambda o2:(-o2.privacyClass, o2.fullName()))
+                      key=lambda o2:(-o2.privacyClass.value, o2.fullName()))
 
     def packageInitTable(self):
         init = self.ob.contents['__init__']
         children = sorted(
-            [o for o in init.orderedcontents if o.isVisible],
-            key=lambda o2:(-o2.privacyClass, o2.fullName()))
+            [o for o in init.contents.values() if o.isVisible],
+            key=lambda o2:(-o2.privacyClass.value, o2.fullName()))
         if children:
             return [tags.p("From the ", tags.code("__init__.py"), " module:",
                            class_="fromInitPy"),
@@ -197,8 +206,8 @@ class PackagePage(CommonPage):
             return ()
 
     def methods(self):
-        return [o for o in self.ob.contents['__init__'].orderedcontents
-                if o.documentation_location == model.DocLocation.PARENT_PAGE
+        return [o for o in self.ob.contents['__init__'].contents.values()
+                if o.documentation_location is model.DocLocation.PARENT_PAGE
                 and o.isVisible]
 
 class ModulePage(CommonPage):
@@ -225,8 +234,9 @@ def nested_bases(b):
 def unmasked_attrs(baselist):
     maybe_masking = set()
     for b in baselist[1:]:
-        maybe_masking.update(set([o.name for o in b.orderedcontents]))
-    return [o for o in baselist[0].orderedcontents if o.name not in maybe_masking]
+        maybe_masking.update(set([o.name for o in b.contents.values()]))
+    return [o for o in baselist[0].contents.values()
+            if o.isVisible and o.name not in maybe_masking]
 
 
 def assembleList(system, label, lst, idbase):
@@ -305,7 +315,7 @@ class ClassPage(CommonPage):
         return [item.clone().fillSlots(
                           baseName=self.baseName(b),
                           baseTable=ChildTable(self.docgetter, self.ob,
-                                               sorted(attrs, key=lambda o:-o.privacyClass)))
+                                               sorted(attrs, key=lambda o:-o.privacyClass.value)))
                 for b, attrs in baselists]
 
     def baseName(self, data):
