@@ -1,16 +1,17 @@
 from __future__ import print_function
 
-from twisted.web.template import Element, renderer, XMLFile
-
 from pydoctor.templatewriter import util
+from twisted.web.template import Element, XMLFile, renderer
+
 
 class AttributeChild(Element):
 
     loader = XMLFile(util.templatefilepath('attribute-child.html'))
 
-    def __init__(self, docgetter, ob):
+    def __init__(self, docgetter, ob, functionExtras):
         self.docgetter = docgetter
         self.ob = ob
+        self._functionExtras = functionExtras
 
     @renderer
     def class_(self, request, tag):
@@ -31,6 +32,17 @@ class AttributeChild(Element):
     def attribute(self, request, tag):
         return self.ob.name
 
+    @renderer
+    def sourceLink(self, request, tag):
+        if self.ob.sourceHref:
+            return tag.fillSlots(sourceHref=self.ob.sourceHref)
+        else:
+            return ()
+
+    @renderer
+    def functionExtras(self, request, tag):
+        return self._functionExtras
+
     @renderer
     def functionBody(self, request, tag):
         return self.docgetter.get(self.ob)
