@@ -3,9 +3,12 @@ Unit tests for model.
 """
 from __future__ import print_function
 
+import sys
+import zlib
+
 from pydoctor import model, sphinx
 from pydoctor.driver import parse_args
-import zlib
+from pydoctor.test.test_astbuilder import fromText
 
 
 class FakeOptions(object):
@@ -100,9 +103,9 @@ def test_fetchIntersphinxInventories_content():
         ]
     url_content = {
         'http://sphinx/objects.inv': zlib.compress(
-            'sphinx.module py:module -1 sp.html -'),
+            b'sphinx.module py:module -1 sp.html -'),
         'file:///twisted/index.inv': zlib.compress(
-            'twisted.package py:module -1 tm.html -'),
+            b'twisted.package py:module -1 tm.html -'),
         }
     sut = model.System(options=options)
     log = []
@@ -121,3 +124,53 @@ def test_fetchIntersphinxInventories_content():
         'file:///twisted/tm.html' ==
         sut.intersphinx.getLink('twisted.package')
         )
+
+
+def test_docsources_class_attribute():
+    src = '''
+    class Base:
+        attr = False
+        """documentation"""
+    class Sub(Base):
+        attr = True
+    '''
+    mod = fromText(src)
+    base_attr = mod.contents['Base'].contents['attr']
+    sub_attr = mod.contents['Sub'].contents['attr']
+    assert base_attr in list(sub_attr.docsources())
+
+
+def test_docstring_lineno():
+    src = '''
+    def f():
+        """
+        This is a long docstring.
+
+        Somewhat long, anyway.
+        This should be enough.
+        """
+    '''
+    mod = fromText(src)
+    func = mod.contents['f']
+    assert func.linenumber == 2
+    assert func.docstring_lineno == 4 # first non-blank line
+
+
+class Dummy(object):
+    def crash(self):
+        """Mmm"""
+
+def test_introspection():
+    """Find docstrings from this test using introspection."""
+    system = model.System()
+    py_mod = sys.modules[__name__]
+    system.introspectModule(py_mod, __name__)
+
+    module = system.objForFullName(__name__)
+    assert module.docstring == __doc__
+
+    func = module.contents['test_introspection']
+    assert func.docstring == "Find docstrings from this test using introspection."
+
+    method = system.objForFullName(__name__ + '.Dummy.crash')
+    assert method.docstring == "Mmm"
