@@ -3,7 +3,6 @@ from __future__ import print_function
 import os
 
 from pydoctor import model
-from pydoctor.test import test_astbuilder
 
 testpackages = os.path.join(os.path.dirname(__file__), 'testpackages')
 
@@ -15,15 +14,14 @@ def processPackage(packname, systemcls=model.System):
     system.process()
     return system
 
-def test_local_import():
-    system = processPackage("localimporttest")
-    cls = system.allobjects['localimporttest.mod1.C']
-    assert len(system.warnings['local import']) > 0
-    assert cls.bases == ['localimporttest.mod2.B']
+def test_relative_import():
+    system = processPackage("relativeimporttest")
+    cls = system.allobjects['relativeimporttest.mod1.C']
+    assert cls.bases == ['relativeimporttest.mod2.B']
 
 def test_package_docstring():
-    system = processPackage("localimporttest")
-    assert (system.allobjects['localimporttest.__init__'].docstring ==
+    system = processPackage("relativeimporttest")
+    assert (system.allobjects['relativeimporttest.__init__'].docstring ==
             "DOCSTRING")
 
 def test_modnamedafterbuiltin():
@@ -39,23 +37,11 @@ def test_nestedconfusion():
     C = system.allobjects['nestedconfusion.mod.C']
     assert A.baseobjects[0] is C
 
-def test_moresystems():
-    system = processPackage("basic")
-    system2 = model.System()
-    system2.moresystems.append(system)
-    mod = test_astbuilder.fromText("""
-    from basic import mod
-    class E(mod.C):
-        pass
-    """, system=system2)
-    E = mod.contents["E"]
-    assert E.baseobjects[0] is not None
-
 def test_importingfrompackage():
     system = processPackage("importingfrompackage")
     system.getProcessedModule('importingfrompackage.mod')
     submod = system.allobjects['importingfrompackage.subpack.submod']
-    assert submod.state == model.PROCESSED
+    assert submod.state is model.ProcessingState.PROCESSED
 
 def test_allgames():
     system = processPackage("allgames")
