@@ -3,28 +3,27 @@ from __future__ import print_function
 import os
 import shutil
 import tempfile
+from io import BytesIO
 
-from twisted.python.compat import NativeStringIO
-
-from pydoctor import templatewriter, model
+import pytest
+from pydoctor import model, templatewriter
 from pydoctor.templatewriter import pages, writer
 from pydoctor.test.test_astbuilder import fromText
 from pydoctor.test.test_packages import processPackage
-import pytest
 
 
 def flatten(t):
-    io = NativeStringIO()
+    io = BytesIO()
     writer.flattenToFile(io, t)
-    return io.getvalue()
+    return io.getvalue().decode()
 
 
 def getHTMLOf(ob):
     wr = templatewriter.TemplateWriter('')
     wr.system = ob.system
-    f = NativeStringIO()
+    f = BytesIO()
     wr.writeDocsForOne(ob, f)
-    return f.getvalue()
+    return f.getvalue().decode()
 
 def test_simple():
     src = '''
@@ -43,7 +42,7 @@ def test_empty_table():
 
 def test_nonempty_table():
     mod = fromText('def f(): pass')
-    t = pages.ChildTable(pages.DocGetter(), mod, mod.orderedcontents)
+    t = pages.ChildTable(pages.DocGetter(), mod, mod.contents.values())
     flattened = flatten(t)
     assert 'The renderer named' not in flattened
 
@@ -76,10 +75,11 @@ def test_basic_package():
         root, = system.rootobjects
         w.writeDocsFor(root, False)
         w.writeModuleIndex(system)
-        for ob in system.allobjects.itervalues():
-            if ob.documentation_location == model.DocLocation.OWN_PAGE:
+        for ob in system.allobjects.values():
+            if ob.documentation_location is model.DocLocation.OWN_PAGE:
                 assert os.path.isfile(os.path.join(targetdir, ob.fullName() + '.html'))
-        assert 'Package docstring' in open(os.path.join(targetdir, 'basic.html')).read()
+        with open(os.path.join(targetdir, 'basic.html')) as f:
+            assert 'Package docstring' in f.read()
     finally:
         shutil.rmtree(targetdir)
 
@@ -91,6 +91,16 @@ def test_hasdocstring():
     sub_f = system.allobjects['basic.mod.D.f']
     assert hasdocstring(sub_f) and not sub_f.docstring
 
+def test_missing_variable():
+    mod = fromText('''
+    """Module docstring.
+
+    @type thisVariableDoesNotExist: Type for non-existent variable.
+    """
+    ''')
+    html = getHTMLOf(mod)
+    assert 'thisVariableDoesNotExist' not in html
+
 
 @pytest.mark.parametrize(
     'className',
@@ -105,7 +115,7 @@ def test_multipleInheritanceNewClass(className):
 
     cls = next(
         cls
-        for cls in system.orderedallobjects
+        for cls in system.allobjects.values()
         if cls.name == className
     )
 
